@@ -1,13 +1,13 @@
-/* @(#)make.c	1.179 10/03/19 Copyright 1985, 87, 88, 91, 1995-2010 J. Schilling */
+/* @(#)make.c	1.185 12/12/20 Copyright 1985, 87, 88, 91, 1995-2012 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)make.c	1.179 10/03/19 Copyright 1985, 87, 88, 91, 1995-2010 J. Schilling";
+	"@(#)make.c	1.185 12/12/20 Copyright 1985, 87, 88, 91, 1995-2012 J. Schilling";
 #endif
 /*
  *	Make program
  *
- *	Copyright (c) 1985, 87, 88, 91, 1995-2010 by J. Schilling
+ *	Copyright (c) 1985, 87, 88, 91, 1995-2012 by J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -43,7 +43,7 @@ static	UConst char sccsid[] =
 
 #include "make.h"
 
-char	make_version[] = "1.2.1";
+char	make_version[] = "1.2.2";
 
 #ifdef	NO_DEFAULTS_PATH
 #undef	DEFAULTS_PATH
@@ -93,16 +93,11 @@ LOCAL	void	handler		__PR((int signo));
 LOCAL	void	exhandler	__PR((int excode, void *arg));
 EXPORT	char	*curwdir	__PR((void));
 LOCAL	char	*getdefaultsfile	__PR((void));
-LOCAL	char	*searchfileinpath	__PR((char *name));
-LOCAL	char	*searchonefile	__PR((char *name, char *nbuf, char *np, char *ep));
 LOCAL	int	put_env		__PR((char *new));
 LOCAL	int	unset_env	__PR((char *name));
 #ifndef	HAVE_PUTENV
 EXPORT	int	putenv		__PR((const char *new));
 #endif
-#if defined(__DJGPP__)
-LOCAL	char 	*strbs2s	__PR((char *s));
-#endif
 #ifndef	HAVE_UNSETENV
 EXPORT	int	unsetenv	__PR((const char *name));
 #endif
@@ -129,7 +124,7 @@ int	Dmake		= 0;		/* -D Display makefile		*/
 BOOL	help		= FALSE;	/* -help    Show Usage		*/
 BOOL	pversion	= FALSE;	/* -version Show version string	*/
 BOOL	No_Warn		= FALSE;	/* -w No warnings		*/
-BOOL	Do_Warn		= FALSE;	/* -W Print extra warnings	*/
+int	Do_Warn		= 0;		/* -W Print extra warnings	*/
 char	Makeflags[]	= "MAKEFLAGS";
 char	Make_Flags[]	= "MAKE_FLAGS";
 char	Make_Macs[]	= "MAKE_MACS";
@@ -260,9 +255,11 @@ usage(exitcode)
 	error("	-q	Question mode. Exit code is 0 if target is up to date.\n");
 	error("	-r	Turn off internal rules.\n");
 	error("	-s	Be silent.\n");
+	error("	-S	Undo the effect of the -k option, terminate on target erros.\n");
 	error("	-t	Touch Objects instead of executing defined commands.\n");
 	error("	-w	Don't print warning Messages.\n");
 	error("	-W	Print extra (debug) warning Messages.\n");
+	error("	-WW	Print even more (debug) warning Messages.\n");
 	error("	-D	Display Makefiles as read in.\n");
 	error("	-DD	Display Makefiles/Rules as read in.\n");
 	error("	-d	Print reason why a target has to be rebuilt.\n");
@@ -645,9 +642,9 @@ setup_SHELL()
 	if (shellname == NULL)
 		shellname = getenv("DJGPP_SH");	/* Backward compat */
 	if (shellname == NULL)
-		shellname = searchfileinpath("bin/sh.exe"); /* alloc() */
+		shellname = searchfileinpath("bin/sh.exe", X_OK, TRUE, NULL); /* alloc() */
 	if (shellname == NULL)
-		shellname = searchfileinpath("sh.exe");	/* alloc() */
+		shellname = searchfileinpath("sh.exe", X_OK, TRUE, NULL);	/* alloc() */
 	if (shellname != NULL)
 		shell = shellname;
 #endif	/* !__DJGPP__ */
@@ -902,7 +899,7 @@ main(ac, av)
 		int	cac = ac;
 		char	* const *cav = av;
 		char	*newdir = NULL;
-	static	char	options[] = "help,version,posix,e,i,k,n,N,p,q,r,s,S,t,w,W,d+,D+,xM,xd+,probj,C*,mf&,f&,&";
+	static	char	options[] = "help,version,posix,e,i,k,n,N,p,q,r,s,S,t,w,W+,d+,D+,xM,xd+,probj,C*,mf&,f&,&";
 
 	save_args(ac, av);
 
@@ -940,7 +937,7 @@ main(ac, av)
 	if (help)
 		usage(0);
 	if (pversion) {
-		printf("Smake release %s (%s-%s-%s) Copyright (C) 1985, 87, 88, 91, 1995-2010 Jrg Schilling\n",
+		printf("Smake release %s (%s-%s-%s) Copyright (C) 1985, 87, 88, 91, 1995-2012 Jrg Schilling\n",
 				make_version,
 				HOST_CPU, HOST_VENDOR, HOST_OS);
 		exit(0);
@@ -966,6 +963,9 @@ main(ac, av)
 		Sflag = TRUE;
 		Nflag = TRUE;
 	}
+	if (Tflag && !Nflag) {
+		Nflag = -1;	/* Hack for touch_file() */
+	}
 	if (Stopflag) {
 		Kflag = FALSE;
 	}
@@ -1244,7 +1244,7 @@ getmakeflags()
 			break;
 
 		case 'W':		/* Extra Warnings */
-			Do_Warn = TRUE;
+			Do_Warn++;
 			break;
 
 		case 'w':		/* No Warnings */
@@ -1376,11 +1376,11 @@ setmakeflags()
 {
 		/*
 		 * MAKEFLAGS=-	12 bytes incl '\0'
-		 * 3 x 8 bytes=	24 bytes
+		 * 4 x 8 bytes=	32 bytes
 		 * 15 flags	15 bytes
 		 * '-- '	 3 bytes
 		 * =====================
-		 *		54 bytes
+		 *		62 bytes
 		 */
 #define	MAKEENV_SIZE_STATIC	64
 static	char	makeenv[MAKEENV_SIZE_STATIC];
@@ -1404,6 +1404,12 @@ static	char	makeenv[MAKEENV_SIZE_STATIC];
 	while (--i >= 0)
 		*p++ = 'd';
 
+	i = Do_Warn;		/* Do_Wan - Extra Warnings */
+	if (i > 8)
+		i = 8;
+	while (--i >= 0)
+		*p++ = 'W';
+
 	i = XDebug;		/* XDebug */
 	if (i > 8)
 		i = 8;
@@ -1434,8 +1440,6 @@ static	char	makeenv[MAKEENV_SIZE_STATIC];
 		*p++ = 'S';
 	if (Tflag)		/* Touch */
 		*p++ = 't';
-	if (Do_Warn)		/* Extra Warnings */
-		*p++ = 'W';
 	if (No_Warn)		/* No Warnings */
 		*p++ = 'w';
 	if (Prdep)		/* Print includes */
@@ -1561,16 +1565,56 @@ setmakeenv(envbase, envp)
 	doexport(Make_Macs);
 }
 
+#include "job.h"
+#define	MAXJ	1
+int	curjobs = 0;
+int	maxjobs = MAXJ;
+job	jobs[MAXJ];
+
+EXPORT	void	cmd_run		__PR((job *jobp));
+EXPORT	int	cmd_wait	__PR((job *jobp));
+
 #define	iswhite(c)	((c) == ' ' || (c) == '\t')
 
 #if	defined(__linux__) || defined(__linux) || \
 	defined(SHELL_IS_BASH) || defined(BIN_SHELL_IS_BASH)
 /*
- * Needed to handle the Linux bash signal bug.
+ * Needed to handle the Linux/bash signal bug.
  */
-int	lpid;
+pid_t	lpid;
 #endif
 
+#ifdef	__needed__
+EXPORT BOOL
+cmd_prefix(cmd, pfx)
+	register char	*cmd;
+	register int	pfx;
+{
+	register char	c;
+
+	while ((c = *cmd++) != '\0') {
+		if (c != '@' && c != '-' && c != '+' && c != '?' && c != '!')
+			break;
+		if (c == pfx)
+			return (TRUE);
+	}
+	return (FALSE);
+}
+
+EXPORT BOOL
+cmdlist_prefix(cmd, pfx)
+	register cmd_t	*cmd;
+	register int	pfx;
+{
+	while (cmd) {
+		if (cmd_prefix(cmd->c_line, pfx))
+			return (TRUE);
+		cmd = cmd->c_next;
+	}
+	return (FALSE);
+}
+#endif /* __needed__ */
+
 /*
  * Execute or print a command line.
  * Return exit code from command line.
@@ -1581,16 +1625,11 @@ docmd(cmd, obj)
 	obj_t		*obj;
 {
 	int	code;
-	int	pid = 0;
-	int	retries;
-	int	Exit;
 	int	Silent = Sflag;
 	int	NoError = Iflag;
 	int	NoExec = Nflag;
 	BOOL	foundplus = FALSE;
-	obj_t	*shello;
-	char	*shell = NULL;
-	char	*shellflag;
+	job	*jobp = NULL;
 
 	while (iswhite(*cmd))
 		cmd++;
@@ -1611,10 +1650,10 @@ docmd(cmd, obj)
 			foundplus = TRUE;
 		}
 		if (*cmd == '?') {
-			/* EMPTY */ ;		/* XXX To be defined !!! */
+			/* EMPTY */;		/* XXX To be defined !!! */
 		}
 		if (*cmd == '!') {
-			/* EMPTY */ ;		/* XXX To be defined !!! */
+			/* EMPTY */;		/* XXX To be defined !!! */
 		}
 	}
 	if (foundplus)
@@ -1631,8 +1670,44 @@ docmd(cmd, obj)
 	if (NoExec && !found_make)
 		return (0);
 
+	jobp = &jobs[0];
+	if (curjobs >= maxjobs) {
+		(void) cmd_wait(jobp);
+	}
+
+	jobp->j_pid = 0;
+	jobp->j_flags = 0;
+	if (Silent)
+		jobp->j_flags |= J_SILENT;
+	if (NoError)
+		jobp->j_flags |= J_NOERROR;
+	jobp->j_cmd = cmd;
+	jobp->j_obj = obj;
+	curjobs++;
+
 	curtarget = obj;
 
+	cmd_run(jobp);
+	return (cmd_wait(jobp));
+}
+
+EXPORT	void	cmd_run	__PR((job *jobp));
+
+EXPORT void
+cmd_run(jobp)
+	job	*jobp;
+{
+	pid_t	pid = 0;
+	int	retries;
+#if defined(USE_SYSTEM) || defined(__DJGPP__) || defined(__MINGW32__) || defined(_MSC_VER)
+	int	Exit;
+#endif
+	int	NoError = (jobp->j_flags & J_NOERROR) != 0;
+	obj_t	*shello;
+	char	*shell = NULL;
+	char	*shellflag;
+	char	*cmd = jobp->j_cmd;
+
 	shellflag = get_var(NoError ? "MAKE_SHELL_IFLAG":"MAKE_SHELL_FLAG");
 	if (shellflag == NULL || *shellflag == '\0') {
 		if (posixmode)
@@ -1652,13 +1727,15 @@ docmd(cmd, obj)
 	(((defined(HAVE_FORK) || defined(HAVE_VFORK)) && \
 		defined HAVE_EXECL) || defined(JOS))
 
-#if defined(__EMX__) || defined(__DJGPP__)
+#if defined(__EMX__) || defined(__DJGPP__) || defined(__MINGW32__) || defined(_MSC_VER)
+
 #ifdef	__EMX__
 	pid = spawnl(P_NOWAIT, shell, filename(shell), shellflag,
 							cmd, (char *)NULL);
 	if (pid < 0)
 		comerr("Can't spawn %s.\n", shell);
-#else
+#else	/* ! __EMX__ */
+
 	if (shello == NULL)
 		comerr("Can't find sh.exe.\n");		/* DJGPP setup problem */
 
@@ -1668,8 +1745,11 @@ docmd(cmd, obj)
 		/* TODO: DOS error code to UNIX error code */
 		Exit = 0xFF<<8;
 	}
-#endif
-#else
+	jobp->j_flags |= J_NOWAIT;
+	jobp->j_excode = Exit;
+#endif	/* ! __EMX__ */
+
+#else	/* ! __EMX__ && ! __DJGPP__ && ! __MINGW32__ */
 	/*
 	 *  Do several tries to fork child to allow working on loaded systems.
 	 */
@@ -1700,7 +1780,8 @@ docmd(cmd, obj)
 							cmd, (char *)NULL);
 		comerr("Can't exec %s.\n", shell);
 	}
-#endif	/* __EMX__ || __DJGPP__ */
+#endif	/* ! __EMX__ && ! __DJGPP__ && ! __MINGW32__ */
+
 
 #if	defined(__linux__) || defined(__linux) || \
 	defined(SHELL_IS_BASH) || defined(BIN_SHELL_IS_BASH)
@@ -1709,15 +1790,61 @@ docmd(cmd, obj)
 	 */
 	lpid = pid;
 #endif
+	jobp->j_pid = pid;
 
-	/* Parent process: wait for child. */
-#ifndef	__DJGPP__
-	if ((code = wait((WAIT_T *)&Exit)) != pid)
-		comerrno(Exit, "code %d waiting for %s.\n", geterrno(), shell);
-#endif
 #else	/* Use system() */
 	Exit = system(cmd);
+	jobp->j_flags |= J_NOWAIT;
+	jobp->j_excode = Exit;
 #endif
+}
+
+EXPORT	int	cmd_wait	__PR((job *jobp));
+
+EXPORT int
+cmd_wait(jobp)
+	job	*jobp;
+{
+	int	code;
+	pid_t	pid = jobp->j_pid;
+	int	Exit;
+	int	Silent = (jobp->j_flags & J_SILENT) != 0;
+	int	NoError = (jobp->j_flags & J_NOERROR) != 0;
+	obj_t	*shello;
+	char	*shell = NULL;
+	char	*cmd = jobp->j_cmd;
+	obj_t	*obj = jobp->j_obj;
+
+	shello = objlook("SHELL", FALSE);
+	shell = get_var("SHELL");
+	if (shell == NULL || *shell == '\0') {
+		shello = NULL;
+		shell = "/bin/sh";
+	}
+
+	if (jobp->j_flags & J_NOWAIT) {
+		Exit = jobp->j_excode;
+		goto nowait;
+	}
+
+	/* Parent process: wait for child. */
+#ifdef	HAVE_WAIT
+	if ((code = wait((WAIT_T *)&Exit)) != pid)
+		comerrno(Exit, "code %d waiting for %s.\n", geterrno(), shell);
+#else
+	/*
+	 * Some platforms (__DJGPP__, __MINGW32__) do not have wait(), but we
+	 * set J_NOWAIT on such platforms and thus should never come here.
+	 * WIN-DOS (__MINGW32__) offers a cwait() that is not complatible to
+	 * cwait() from UNOS, so WIN-DOS created a clash.
+	 */
+	/* NOTREACHED */
+	errmsgno(EX_BAD,
+	"Implementation botch, wait for %s not possible on this platform.\n",
+			shell);
+#endif
+
+nowait:
 	if (Exit) {
 		errmsgno(Exit>>8, "*** Code %d from command line for target '%s'%s.\n",
 			Exit>>8,
@@ -1728,6 +1855,9 @@ docmd(cmd, obj)
 			error("%s\n", cmd);
 		}
 	}
+	curjobs--;
+	jobp->j_pid = 0;
+	jobp->j_obj = (obj_t *)NULL;
 	curtarget = (obj_t *)NULL;
 
 	if (NoError)
@@ -1867,13 +1997,17 @@ copy_file(from, objname)
 		if ((fout = creat(objname, 0666)) < 0)
 			errmsg("Can't create '%s'.\n", objname);
 		else {
-			while ((cnt = read(fin, gbuf, gbufsize)) > 0)
+			while ((cnt = read(fin, gbuf, gbufsize)) > 0) {
 				if (write(fout, gbuf, cnt) != cnt) {
 					errmsg("Write error on '%s'.\n",
 								objname);
-					cnt = -1;
-					break;
+					close(fout);
+					close(fin);
+					return (-1);
 				}
+			}
+			if (cnt < 0)
+				errmsg("Read error on '%s'.\n", from);
 			close(fout);
 		}
 		close(fin);
@@ -1896,8 +2030,19 @@ copy_file(from, objname)
 		if ((fout = fileopen(objname, "wtcub")) == 0)
 			errmsg("Can't create '%s'.\n", objname);
 		else {
-			while ((cnt = fileread(fin, gbuf, gbufsize)) > 0)
-				filewrite(fout, gbuf, cnt);
+			file_raise(fin, FALSE);
+			file_raise(fout, FALSE);
+			while ((cnt = fileread(fin, gbuf, gbufsize)) > 0) {
+				if (filewrite(fout, gbuf, cnt) < 0) {
+					errmsg("Write error on '%s'.\n",
+								objname);
+					fclose(fout);
+					fclose(fin);
+					return (-1);
+				}
+			}
+			if (cnt < 0)
+				errmsg("Read error on '%s'.\n", from);
 			fclose(fout);
 		}
 		fclose(fin);
@@ -1937,9 +2082,13 @@ again:
 #endif
 	if (!Sflag)
 		printf("%stouch %s\n", posixmode?"\t":"...", objname);
-	if (Nflag)
+	if (Nflag > 0)
 		return (TRUE);
+	/*
+	 * No touch if make was called with -n.
+	 */
 	if ((f = fileopen(objname, "rwcub")) != (FILE *)NULL) {
+		file_raise(f, FALSE);
 #ifdef	HAVE_UTIME
 		utime(objname, NULL);
 #else
@@ -2015,6 +2164,19 @@ gftime(file)
 	char	this_time[32];
 	char	cur_time[32];
 
+	/*
+	 * XXX should we cache the file times?
+	 */
+/*#define	nonono__*/
+#ifdef	nonono__
+{
+	obj_t	*o = objlook(file, FALSE);
+
+	if (o != NULL && VALIDTIME(o->o_date))
+		return (o->o_date);		/* Cached file time */
+}
+#endif
+
 	stbuf.st_mtime = NOTIME;
 	if (stat(file, &stbuf) < 0) {
 		/*
@@ -2272,85 +2434,7 @@ curwdir()
 LOCAL char *
 getdefaultsfile()
 {
-	return (searchfileinpath("lib/defaults.smk"));
-}
-
-LOCAL char *
-searchfileinpath(name)
-	char	*name;
-{
-	char	*path = getenv("PATH");
-	char	pbuf[NAMEMAX];
-	char	*nbuf = pbuf;
-	char	*np;
-	int	nlen = strlen(name);
-#ifdef	HAVE_GETEXECNAME
-	char	*pn = (char *)getexecname();
-#else
-	char	*pn = get_progname();
-#endif
-
-	if (strchr(pn, '/') != NULL) {
-		strncpy(nbuf, pn, sizeof (pbuf));
-		nbuf[sizeof (pbuf) - 1] = '\0';
-		np = nbuf + strlen(nbuf);
-
-		while (np > nbuf && np[-1] != '/')
-			*--np = '\0';
-		pn = &nbuf[sizeof (pbuf) - 1];
-		if ((np = searchonefile(name, nbuf, np, pn)) != NULL)
-			return (np);
-	}
-
-	if (path == NULL)
-		return (NULL);
-
-
-#ifdef __DJGPP__
-	strbs2s(path);	/* PATH under DJGPP can contain both slashes */
-#endif
-
-	pn = &nbuf[sizeof (pbuf) - 1];
-	for (;;) {
-		np = nbuf;
-		while (*path != PATH_ENV_DELIM && *path != '\0' &&
-		    np < &nbuf[sizeof (pbuf) - nlen])
-				*np++ = *path++;
-		*np = '\0';
-		if ((np = searchonefile(name, nbuf, np, pn)) != NULL)
-			return (np);
-
-		if (*path == '\0')
-			break;
-		path++;
-	}
-	return (NULL);
-}
-
-LOCAL char *
-searchonefile(name, nbuf, np, ep)
-	register char	*name;
-	register char	*nbuf;
-	register char	*np;
-	register char	*ep;
-{
-	while (np > nbuf && np[-1] == '/')
-		*--np = '\0';
-	if (np >= &nbuf[4] && streql(&np[-4], "/bin"))
-		np = &np[-4];
-	*np++ = '/';
-	*np   = '\0';
-	strncpy(np, name, ep - np);
-	*ep = '\0';
-
-	if (gftime(nbuf)) {
-		np = malloc(strlen(nbuf)+1);
-		if (np == NULL)
-			return (NULL);
-		strcpy(np, nbuf);
-		return (np);
-	}
-	return (NULL);
+	return (searchfileinpath("lib/defaults.smk", R_OK, TRUE, NULL));
 }
 
 LOCAL int
@@ -2374,24 +2458,6 @@ unset_env(name)
 	return (0);
 }
 
-#ifdef __DJGPP__
-LOCAL char *
-strbs2s(s)
-	char	*s;
-{
-	char	*tmp = s;
-
-	if (tmp) {
-		while (*tmp) {
-			if (*tmp == '\\')
-				*tmp = '/';
-			tmp++;
-		}
-	}
-	return (s);
-}
-#endif
-
 #ifndef	HAVE_PUTENV
 
 EXPORT	int	putenv		__PR((const char *new));
@@ -2516,4 +2582,3 @@ ovstrcpy(p2, p1)
 	while ((*p2++ = *p1++) != '\0')
 		;
 }
-
